package org.apfloat.internal;

import org.apfloat.*;
import org.apfloat.spi.*;

import junit.framework.TestSuite;

/**
 * @version 1.6
 * @author Mikko Tommila
 */

public class Double3NTTConvolutionStrategyTest
    extends DoubleConvolutionStrategyTestCase
    implements DoubleRadixConstants
{
    public Double3NTTConvolutionStrategyTest(String methodName)
    {
        super(methodName);
    }

    public static void main(String[] args)
    {
        junit.textui.TestRunner.run(suite());
    }

    public static TestSuite suite()
    {
        TestSuite suite = new TestSuite();

        suite.addTest(new Double3NTTConvolutionStrategyTest("testFull"));
        suite.addTest(new Double3NTTConvolutionStrategyTest("testTruncated"));
        suite.addTest(new Double3NTTConvolutionStrategyTest("testAuto"));
        suite.addTest(new Double3NTTConvolutionStrategyTest("testFullBig"));
        suite.addTest(new Double3NTTConvolutionStrategyTest("testFullBigParallel"));
        suite.addTest(new Double3NTTConvolutionStrategyTest("testTruncatedBig"));
        suite.addTest(new Double3NTTConvolutionStrategyTest("testAutoBig"));
        suite.addTest(new Double3NTTConvolutionStrategyTest("testAutoBigParallel"));

        return suite;
    }

    public static void testFull()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            double b1 = BASE[radix] - (double) 1;
            DataStorage src1 = createDataStorage(new double[] { (double) 1, (double) 2, (double) 3, (double) 4 }),
                        src2 = createDataStorage(new double[] { (double) 5, (double) 6 }),
                        src9 = createDataStorage(new double[] { b1, b1, b1 }),
                        src99 = createDataStorage(new double[] { b1, b1, b1, b1, b1, b1 });

            ConvolutionStrategy convolutionStrategy = new Double3NTTConvolutionStrategy(radix, new DoubleTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src1, src2, 6);

            check("normal", radix, new double[] { 0, (double) 5, (double) 16, (double) 27, (double) 38, (double) 24 }, result);

            result = convolutionStrategy.convolute(src9, src99, 9);

            check("max", radix, new double[] { b1, b1, b1 - (double) 1, b1, b1, b1, 0, 0, (double) 1 }, result);
        }
    }

    public static void testTruncated()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            double b1 = BASE[radix] - (double) 1;
            DataStorage src1 = createDataStorage(new double[] { (double) 1, (double) 2, (double) 3, (double) 4 }),
                        src2 = createDataStorage(new double[] { (double) 5, (double) 6 }),
                        src9 = createDataStorage(new double[] { b1, b1, b1 }),
                        src99 = createDataStorage(new double[] { b1, b1, b1, b1, b1, b1 });

            ConvolutionStrategy convolutionStrategy = new Double3NTTConvolutionStrategy(radix, new DoubleTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src1, src2, 3);

            check("normal", radix, new double[] { 0, (double) 5, (double) 16 }, result);

            result = convolutionStrategy.convolute(src9, src99, 6);

            check("max", radix, new double[] { b1, b1, b1 - (double) 1, b1, b1, b1 }, result);
        }
    }

    public static void testAuto()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            double b1 = BASE[radix] - (double) 1;
            DataStorage src9 = createDataStorage(new double[] { b1, b1, b1 });

            ConvolutionStrategy convolutionStrategy = new Double3NTTConvolutionStrategy(radix, new DoubleTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src9, src9, 6);

            check("max", radix, new double[] { b1, b1, b1 - (double) 1, 0, 0, (double) 1 }, result);
        }
    }

    public static void testFullBig()
    {
        ApfloatContext ctx = ApfloatContext.getContext();
        int numberOfProcessors = ctx.getNumberOfProcessors();
        ctx.setNumberOfProcessors(1);

        runBig();

        ctx.setNumberOfProcessors(numberOfProcessors);
    }

    public static void testFullBigParallel()
    {
        ApfloatContext ctx = ApfloatContext.getContext();
        int numberOfProcessors = ctx.getNumberOfProcessors();
        ctx.setNumberOfProcessors(4);

        runBig();

        ctx.setNumberOfProcessors(numberOfProcessors);
    }

    private static void runBig()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            double b1 = BASE[radix] - (double) 1;
            int size1 = 500,
                size2 = 300;
            double[] array1 = new double[size1],
                      array2 = new double[size2],
                      array3 = new double[size1 + size2];
            for (int i = 0; i < size1; i++)
            {
                array1[i] = b1;
                array3[i] = b1 - (double) (i == size2 - 1 ? 1 : 0);
            }
            for (int i = 0; i < size2; i++)
            {
                array2[i] = b1;
                array3[i + size1] = (double) (i == size2 - 1 ? 1 : 0);
            }
            DataStorage src9 = createDataStorage(array1),
                        src99 = createDataStorage(array2);

            ConvolutionStrategy convolutionStrategy = new Double3NTTConvolutionStrategy(radix, new DoubleSixStepFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src9, src99, size1 + size2);

            check("max", radix, array3, result);
        }
    }

    public static void testTruncatedBig()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            double b1 = BASE[radix] - (double) 1;
            int size1 = 500,
                size2 = 300;
            double[] array1 = new double[size1],
                      array2 = new double[size2],
                      array3 = new double[size1];
            for (int i = 0; i < size1; i++)
            {
                array1[i] = b1;
                array3[i] = b1 - (double) (i == size1 - 1 || i == size2 - 1 ? 1 : 0);
            }
            for (int i = 0; i < size2; i++)
            {
                array2[i] = b1;
            }
            DataStorage src9 = createDataStorage(array1),
                        src99 = createDataStorage(array2);

            ConvolutionStrategy convolutionStrategy = new Double3NTTConvolutionStrategy(radix, new DoubleTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src99, src9, size1);

            check("max", radix, array3, result);
        }
    }

    public static void testAutoBig()
    {
        ApfloatContext ctx = ApfloatContext.getContext();
        int numberOfProcessors = ctx.getNumberOfProcessors();
        ctx.setNumberOfProcessors(1);

        runAutoBig();

        ctx.setNumberOfProcessors(numberOfProcessors);
    }

    public static void testAutoBigParallel()
    {
        ApfloatContext ctx = ApfloatContext.getContext();
        int numberOfProcessors = ctx.getNumberOfProcessors();
        ctx.setNumberOfProcessors(4);

        runAutoBig();

        ctx.setNumberOfProcessors(numberOfProcessors);
    }

    public static void runAutoBig()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            double b1 = BASE[radix] - (double) 1;
            int size = 500;
            double[] array1 = new double[size],
                      array2 = new double[2 * size];
            for (int i = 0; i < size; i++)
            {
                array1[i] = b1;
                array2[i] = b1 - (double) (i == size - 1 ? 1 : 0);
                array2[i + size] = (double) (i == size - 1 ? 1 : 0);
            }
            DataStorage src9 = createDataStorage(array1);

            ConvolutionStrategy convolutionStrategy = new Double3NTTConvolutionStrategy(radix, new DoubleTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src9, src9, 2 * size);

            check("max", radix, array2, result);
        }
    }
}
